from copy import copy
from worker.worker import Worker

from netboy.multi_pycurl.curl_one import work as curl_work


class NetBoy:
    def __init__(self, info=None):
        self.info = info if info else {}
        self.info['dummy'] = 'netboy.celery.tasks.dummy'
        self.info['log'] = 'netboy'

    def use_socks5_proxy(self, proxy):
        if proxy:
            p = proxy.split(':')
            self.info['proxytype'] = 'socks5'
            self.info['proxy'] = p[0]
            self.info['proxyport'] = int(p[1])
        return self

    def use_http_proxy(self, proxy):
        if proxy:
            p = proxy.split(':')
            self.info['proxytype'] = 'http'
            self.info['proxy'] = p[0]
            self.info['proxyport'] = int(p[1])
        return self

    def use_queue(self, queue):
        self.info['queue'] = queue
        return self

    def use_logger(self, log_name):
        self.info['log'] = log_name
        return self

    def use_filter(self, result_filter):
        self.info['filter'] = result_filter
        return self

    def use_prepares(self, prepares):
        self.info['prepares'] = prepares
        return self

    def use_triggers(self, triggers):
        self.info['triggers'] = triggers
        return self

    def use_analysers(self, analysers):
        self.info['analysers'] = analysers
        return self

    def use_auth(self, user, password, group='default'):
        if user and password:
            self.info['auth'] = {
                'user': user,
                'password': password,
                'group': group
            }
        return self

    def use_useragent(self, useragent):
        self.info['useragent'] = useragent
        return self

    def use_headers(self, headers):
        if isinstance(headers, list):
            self.info['httpheader'] = headers
        elif isinstance(headers, dict):
            http_header = []
            for k,v in headers.items():
                http_header.append(k.replace('_','-')+': '+v)
        return self

    def use_cookies(self, cookies):
        if isinstance(cookies, str):
            self.info['cookie'] = cookies
        elif isinstance(cookies, list):
            self.info['cookie'] = ';'.join(cookies)
        elif isinstance(cookies, dict):
            cookie_strs = []
            for k, v in cookies.items():
                cookie_strs.append(k+'='+v)
            self.info['cookie'] = ';'.join(cookie_strs)
        return self

    def use_timeout(self, timeout=None, connect=None, wait=None, script=None):
        if timeout:
            self.info['timeout'] = timeout
        if connect:
            self.info['connecttimeout'] = connect
        if wait:
            self.info['wait'] = wait
        if script:
            self.info['script_timeout'] = script
        return self

    def use_info(self, info):
        if info:
            self.info = info
        return self

    def use_final(self, final):
        self.info['final'] = final
        return self

    def use_mode(self, mode):
        self.info['mode'] = mode
        spider = self.info.get('spider')
        if spider:
            self.use_spider(spider)
        else:
            if mode == 'coroutine':
                self.use_spider('aiohttp')
        return self

    def use_spider(self, spider='pycurl'):
        self.info['spider'] = spider
        mode = self.info.get('mode', 'thread')
        if mode == 'celery':
            if spider == 'pycurl':
                self.info['celery_worker'] = 'netboy.celery.tasks.pycurl_worker'
                self.info['worker'] = 'netboy.celery.tasks.multicurl_worker_do_crawl'
                self.info['final_callback'] = 'netboy.celery.tasks.final_callback'
            elif spider == 'chrome':
                self.info['celery_worker'] = 'netboy.celery.tasks.thread_worker'
                self.info['worker'] = 'netboy.celery.tasks.chrome_worker_do_crawl'
                self.info['final_callback'] = 'netboy.celery.tasks.final_callback'
        elif mode == 'coroutine' and spider == 'aiohttp':
            self.info['worker'] = 'netboy.aio_http.aiohttp_handler.aiohttp_handler'
        else:
            if spider == 'pycurl':
                self.info['worker'] = 'netboy.multi_pycurl.multicurl_handler.curl_handler'
            elif spider == 'chrome':
                self.info['worker'] = 'netboy.selenium_chrome.chrome_driver_handler.chrome_driver_handler'
            elif spider == 'aiohttp':
                self.info['worker'] = 'netboy.aio_http.aiohttp_handler.aio_http_handler'
        return self

    def use_workers(self, workers=8, chunk_size1=40, chunk_size2=8):
        self.info['celery_max_workers'] = workers
        self.info['max_workers'] = workers
        self.info['celery_chunk_size'] = max(chunk_size1, chunk_size2)
        self.info['chunk_size'] = min(chunk_size1, chunk_size2)
        return self

    def use_logger(self, logger):
        self.info['log'] = logger
        return self

    def use_chrome(self, chrome):
        self.info['chrome'] = chrome
        self.use_spider('chrome')
        return self

    def use_window(self, window):
        self.info['window_size'] = window
        self.use_spider('chrome')
        return self

    def use_maxredirs(self, maxredirs=10):
        if maxredirs:
            self.info['maxredirs'] = maxredirs
        return self

    def use_followlocation(self, followlocation=1):
        if followlocation:
            self.info['followlocation'] = followlocation
        return self

    def use_postfields(self, data, method='post'):
        self.info['method'] = method
        self.info['postfields'] = data

    def run(self, data):
        self.worker = Worker(mode=self.info.get('mode', 'thread'))
        resp = self.worker.work(data, self.info)
        return resp

    def run_remote(self, url, data, callback_data=None):
        self.worker = Worker(mode=self.info.get('mode', 'thread'))
        triggers = self.info.get('triggers')
        trigger_payload = {'trigger': 'netboy.support.triggers.post_it'}
        if callback_data:
            trigger_payload.update(callback_data)
            if triggers:
                self.info['triggers'].append(trigger_payload)
            else:
                self.info['triggers'] = [trigger_payload]

        payload = {
            'url': url,
            'method': 'post',
            'postfields': {
                'info': copy(self.info),
                'data': data
            }
        }

        resp = curl_work(payload, logger='netboy')
        return resp

    def register_remote(self, url, user, password, group='default'):
        payload = {
            'url': url,
            'method': 'post',
            'postfields': {
                'user': user,
                'password': password,
                'group': group
            }
        }

        resp = curl_work(payload, logger='netboy')
        return resp

def write_test(chunk, data, fd=None):
    fd.write(chunk)


if __name__ == '__main__':

    # info={'charset': 'gb2312', 'filter':['title', 'url', 'charset', 'code', 'method', 'headers', 'cookies', 'time']}
    # data=['http://www.bing.com','http://www.csdn.net']#, 'http://www.google.com' ]
    info = {
        'stream': {
            'func': 'netboy.netboy.write_test',
            'file': 'test.jpg'
        }
    }
    # data=['http://www.bing.com','http://www.csdn.net']#, 'http://www.google.com' ]
    data=['https://img3.doubanio.com/view/status/raw/public/ca19211898467d3.jpg']#, 'http://www.google.com' ]
    # f = AIOHttpFactory(data, info)
    # f.run()
    boy = NetBoy(info)
    boy.use_mode('coroutine').use_spider('aiohttp')
    resp = boy.run(data)
    print(resp)


    # data = [
    #            # 'http://www.xixiaagri.gov.cn/'
    #            # 'http://www.xxdsyjs.com'
    #            # 'http://www.puyangdangshi.com'
    #
    #
    #
    #            # 'http://www.csdn.net',
    #            'http://www.bing.com',
    #            # 'http://www.douban.com',
    #            # 'http://www.qxjtzf.com',
    #            # 'http://www.lyzbj.org.cn',
    #            # 'http://www.hnhxrs.com',
    #            # 'http://www.puyangdangshi.com',
    #            # 'http://www.bfhbj.com',
    #            # 'http://www.xxlyj.cn',
    #            # 'http://www.xcsnks.cn',
    #            # 'http://www.xcswmw.cn',
    #            # 'http://www.xcsqxj.com',
    #            # 'http://www.hnpopss.gov.cn',
    #            # 'http://www.lyjtj.com',
    #            # 'http://www.rndj.com',
    #            # 'http://www.nxzj.com.cn',
    #            # 'http://www.ycxyw.gov.cn',
    #            # 'http://www.hnyssw.gov.cn',
    #            # 'http://www.hbcs.gov.cn',
    #            # 'http://www.hnxcdj.com',
    #            # 'http://www.ryzj.gov.cn',
    #            # 'http://www.lhsajj.com',
    #            # 'http://www.nysylj.com',
    #            # 'http://www.nyszglc.com',
    #            # 'http://www.ayjtj.gov.cn',
    #            # 'http://www.hbxgtzyj.gov.cn',
    #            # 'http://www.hnrdia.com',
    #            # 'http://www.zmdggjy.com',
    #            # 'http://www.xyxgtzyj.gov.cn',
    #            # 'http://www.xxdsyjs.com',
    #            # 'http://www.xixiaagri.gov.cn',
    #            # 'http://www.xmdj.gov.cn',
    #            # 'http://www.xysrsjzlzpksbmw.gov.cn',
    #
    #        ] * 1
    # boy = NetBoy()
    # boy.use_analysers([
    #     'netboy.support.analysers.analyse_it'
    # ]).use_queue(
    #     'worker'
    # ).use_spider(
    #     'pycurl'
    # ).use_filter(['url', 'title']).use_workers()
    # resp = boy.run(data)
    # # print(resp)
